@@ -72,11 +72,11 @@ pub(crate) fn handle_msg(
         WM_XBUTTONUP => handle_xbutton_msg(wparam, lparam, handle_mouse_up_msg, state_ptr),
         WM_MOUSEWHEEL => handle_mouse_wheel_msg(handle, wparam, lparam, state_ptr),
         WM_MOUSEHWHEEL => handle_mouse_horizontal_wheel_msg(handle, wparam, lparam, state_ptr),
-        WM_SYSKEYDOWN => handle_syskeydown_msg(handle, wparam, lparam, state_ptr),
-        WM_SYSKEYUP => handle_syskeyup_msg(handle, wparam, state_ptr),
-        WM_KEYDOWN => handle_keydown_msg(handle, wparam, lparam, state_ptr),
-        WM_KEYUP => handle_keyup_msg(handle, wparam, state_ptr),
-        WM_CHAR => handle_char_msg(handle, wparam, lparam, state_ptr),
+        WM_SYSKEYDOWN => handle_syskeydown_msg(wparam, lparam, state_ptr),
+        WM_SYSKEYUP => handle_syskeyup_msg(wparam, state_ptr),
+        WM_KEYDOWN => handle_keydown_msg(wparam, lparam, state_ptr),
+        WM_KEYUP => handle_keyup_msg(wparam, state_ptr),
+        WM_CHAR => handle_char_msg(wparam, lparam, state_ptr),
         WM_IME_STARTCOMPOSITION => handle_ime_position(handle, state_ptr),
         WM_IME_COMPOSITION => handle_ime_composition(handle, lparam, state_ptr),
         WM_SETCURSOR => handle_set_cursor(lparam, state_ptr),
@@ -179,15 +179,13 @@ fn handle_timer_msg(
 }
 
 fn handle_paint_msg(handle: HWND, state_ptr: Rc<WindowsWindowStatePtr>) -> Option<isize> {
-    let mut paint_struct = PAINTSTRUCT::default();
-    let _hdc = unsafe { BeginPaint(handle, &mut paint_struct) };
     let mut lock = state_ptr.state.borrow_mut();
     if let Some(mut request_frame) = lock.callbacks.request_frame.take() {
         drop(lock);
         request_frame();
         state_ptr.state.borrow_mut().callbacks.request_frame = Some(request_frame);
     }
-    unsafe { EndPaint(handle, &paint_struct).ok().log_err() };
+    unsafe { ValidateRect(handle, None).ok().log_err() };
     Some(0)
 }
 
@@ -261,7 +259,6 @@ fn handle_mouse_move_msg(
 }
 
 fn handle_syskeydown_msg(
-    handle: HWND,
     wparam: WPARAM,
     lparam: LPARAM,
     state_ptr: Rc<WindowsWindowStatePtr>,
@@ -281,7 +278,6 @@ fn handle_syskeydown_msg(
         is_held: lparam.0 & (0x1 << 30) > 0,
     };
     let result = if func(PlatformInput::KeyDown(event)).default_prevented {
-        invalidate_client_area(handle);
         Some(0)
     } else {
         None
@@ -291,11 +287,7 @@ fn handle_syskeydown_msg(
     result
 }
 
-fn handle_syskeyup_msg(
-    handle: HWND,
-    wparam: WPARAM,
-    state_ptr: Rc<WindowsWindowStatePtr>,
-) -> Option<isize> {
+fn handle_syskeyup_msg(wparam: WPARAM, state_ptr: Rc<WindowsWindowStatePtr>) -> Option<isize> {
     // we need to call `DefWindowProcW`, or we will lose the system-wide `Alt+F4`, `Alt+{other keys}`
     // shortcuts.
     let Some(keystroke) = parse_syskeydown_msg_keystroke(wparam) else {
@@ -308,7 +300,6 @@ fn handle_syskeyup_msg(
     drop(lock);
     let event = KeyUpEvent { keystroke };
     let result = if func(PlatformInput::KeyUp(event)).default_prevented {
-        invalidate_client_area(handle);
         Some(0)
     } else {
         Some(1)
@@ -319,7 +310,6 @@ fn handle_syskeyup_msg(
 }
 
 fn handle_keydown_msg(
-    handle: HWND,
     wparam: WPARAM,
     lparam: LPARAM,
     state_ptr: Rc<WindowsWindowStatePtr>,
@@ -337,7 +327,6 @@ fn handle_keydown_msg(
         is_held: lparam.0 & (0x1 << 30) > 0,
     };
     let result = if func(PlatformInput::KeyDown(event)).default_prevented {
-        invalidate_client_area(handle);
         Some(0)
     } else {
         Some(1)
@@ -347,11 +336,7 @@ fn handle_keydown_msg(
     result
 }
 
-fn handle_keyup_msg(
-    handle: HWND,
-    wparam: WPARAM,
-    state_ptr: Rc<WindowsWindowStatePtr>,
-) -> Option<isize> {
+fn handle_keyup_msg(wparam: WPARAM, state_ptr: Rc<WindowsWindowStatePtr>) -> Option<isize> {
     let Some(keystroke) = parse_keydown_msg_keystroke(wparam) else {
         return Some(1);
     };
@@ -362,7 +347,6 @@ fn handle_keyup_msg(
     drop(lock);
     let event = KeyUpEvent { keystroke };
     let result = if func(PlatformInput::KeyUp(event)).default_prevented {
-        invalidate_client_area(handle);
         Some(0)
     } else {
         Some(1)
@@ -373,7 +357,6 @@ fn handle_keyup_msg(
 }
 
 fn handle_char_msg(
-    handle: HWND,
     wparam: WPARAM,
     lparam: LPARAM,
     state_ptr: Rc<WindowsWindowStatePtr>,
@@ -396,7 +379,6 @@ fn handle_char_msg(
     let mut lock = state_ptr.state.borrow_mut();
     lock.callbacks.input = Some(func);
     if dispatch_event_result.default_prevented || !dispatch_event_result.propagate {
-        invalidate_client_area(handle);
         return Some(0);
     }
     let Some(ime_char) = ime_key else {
@@ -407,7 +389,6 @@ fn handle_char_msg(
     };
     drop(lock);
     input_handler.replace_text_in_range(None, &ime_char);
-    invalidate_client_area(handle);
     state_ptr.state.borrow_mut().input_handler = Some(input_handler);
 
     Some(0)
@@ -648,7 +629,6 @@ fn handle_ime_composition(
         drop(lock);
         input_handler.replace_text_in_range(None, &comp_result);
         state_ptr.state.borrow_mut().input_handler = Some(input_handler);
-        invalidate_client_area(handle);
         return Some(0);
     }
     // currently, we don't care other stuff
@@ -771,7 +751,6 @@ fn handle_dpi_changed_msg(
         .context("unable to set window position after dpi has changed")
         .log_err();
     }
-    invalidate_client_area(handle);
 
     Some(0)
 }
@@ -1161,12 +1140,6 @@ fn parse_char_msg_keystroke(wparam: WPARAM) -> Option<Keystroke> {
     }
 }
 
-/// mark window client rect to be re-drawn
-/// https://learn.microsoft.com/en-us/windows/win32/api/winuser/nf-winuser-invalidaterect
-pub(crate) fn invalidate_client_area(handle: HWND) {
-    unsafe { InvalidateRect(handle, None, FALSE).ok().log_err() };
-}
-
 fn parse_ime_compostion_string(handle: HWND) -> Option<(String, usize)> {
     unsafe {
         let ctx = ImmGetContext(handle);
