@@ -602,13 +602,6 @@ pub struct LanguageConfig {
     /// or a whole-word search in buffer search.
     #[serde(default)]
     pub word_characters: HashSet<char>,
-    /// The name of a Prettier parser that should be used for this language.
-    #[serde(default)]
-    pub prettier_parser_name: Option<String>,
-    /// The names of any Prettier plugins that should be used for this language.
-    #[serde(default)]
-    pub prettier_plugins: Vec<Arc<str>>,
-
     /// Whether to indent lines using tab characters, as opposed to multiple
     /// spaces.
     #[serde(default)]
@@ -700,8 +693,6 @@ impl Default for LanguageConfig {
             scope_opt_in_language_servers: Default::default(),
             overrides: Default::default(),
             word_characters: Default::default(),
-            prettier_parser_name: None,
-            prettier_plugins: Default::default(),
             collapsed_placeholder: Default::default(),
             hard_tabs: Default::default(),
             tab_size: Default::default(),
@@ -1375,14 +1366,6 @@ impl Language {
         }
     }
 
-    pub fn prettier_parser_name(&self) -> Option<&str> {
-        self.config.prettier_parser_name.as_deref()
-    }
-
-    pub fn prettier_plugins(&self) -> &Vec<Arc<str>> {
-        &self.config.prettier_plugins
-    }
-
     pub fn lsp_id(&self) -> String {
         match self.config.name.as_ref() {
             "Plain Text" => "plaintext".to_string(),
