@@ -144,6 +144,7 @@ impl OutlineViewDelegate {
                 active_editor.highlight_rows::<OutlineRowHighlights>(
                     outline_item.range.start..=outline_item.range.end,
                     Some(cx.theme().colors().editor_highlighted_line_background),
+                    true,
                     cx,
                 );
                 active_editor.request_autoscroll(Autoscroll::center(), cx);
@@ -316,7 +317,7 @@ impl PickerDelegate for OutlineViewDelegate {
 
 #[cfg(test)]
 mod tests {
-    use collections::HashSet;
+    use super::*;
     use gpui::{TestAppContext, VisualTestContext};
     use indoc::indoc;
     use language::{Language, LanguageConfig, LanguageMatcher};
@@ -324,8 +325,6 @@ mod tests {
     use serde_json::json;
     use workspace::{AppState, Workspace};
 
-    use super::*;
-
     #[gpui::test]
     async fn test_outline_view_row_highlights(cx: &mut TestAppContext) {
         init_test(cx);
@@ -484,7 +483,7 @@ mod tests {
     fn highlighted_display_rows(editor: &View<Editor>, cx: &mut VisualTestContext) -> Vec<u32> {
         editor.update(cx, |editor, cx| {
             editor
-                .highlighted_display_rows(HashSet::default(), cx)
+                .highlighted_display_rows(cx)
                 .into_keys()
                 .map(|r| r.0)
                 .collect()
