@@ -112,7 +112,19 @@ macos() {
     ditto "$temp/mount/$app" "/Applications/$app"
     hdiutil detach -quiet "$temp/mount"
 
-    echo "Zed has been installed. Run with 'open /Applications/$app'"
+    mkdir -p "$HOME/.local/bin"
+    # Link the binary
+    ln -sf /Applications/$app/Contents/MacOS/cli "$HOME/.local/bin/zed"
+
+    if which "zed" >/dev/null 2>&1; then
+        echo "Zed has been installed. Run with 'zed'"
+    else
+        echo "To run Zed from your terminal, you must add ~/.local/bin to your PATH"
+        echo "Run:"
+        echo "   echo 'export PATH=\$HOME/.local/bin:\$PATH' >> ~/.bashrc"
+        echo "   source ~/.bashrc"
+        echo "To run Zed now, '~/.local/bin/zed'"
+    fi
 }
 
 main "$@"
